from sanic.response import json


class Handler:
    default_message_key = "msg"

    no_authorization = staticmethod(
        lambda r, e: json({Handler.default_message_key: str(e)}, 401)
    )
    expired_signature = staticmethod(
        lambda r, e: json({Handler.default_message_key: str(e)}, 401)
    )
    invalid_header = staticmethod(
        lambda r, e: json({Handler.default_message_key: str(e)}, 422)
    )
    invalid_token = staticmethod(
        lambda r, e: json({Handler.default_message_key: str(e)}, 422)
    )
    jwt_decode_error = staticmethod(
        lambda r, e: json({Handler.default_message_key: str(e)}, 422)
    )
    wrong_token = staticmethod(
        lambda r, e: json({Handler.default_message_key: str(e)}, 422)
    )
    revoked_token = staticmethod(
        lambda r, e: json({Handler.default_message_key: str(e)}, 401)
    )
    fresh_token_required = staticmethod(
        lambda r, e: json({Handler.default_message_key: str(e)}, 401)
    )
    access_denied = staticmethod(
        lambda r, e: json({Handler.default_message_key: str(e)}, 403)
    )



# Stubs for handler (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.
from typing import Any, Callable, Union

from sanic.request import Request
from sanic.response import HTTPResponse

HandlerSignature = Callable[[Request, Exception], Union[HTTPResponse, Any]]

class Handler:
    default_message_key: str = ...

    no_authorization: HandlerSignature = ...
    expired_signature: HandlerSignature = ...
    invalid_header: HandlerSignature = ...
    invalid_token: HandlerSignature = ...
    jwt_decode_error: HandlerSignature = ...
    wrong_token: HandlerSignature = ...
    revoked_token: HandlerSignature = ...
    fresh_token_required: HandlerSignature = ...
    access_denied: HandlerSignature = ...



import datetime
import json
import uuid
from dataclasses import asdict, dataclass, field
from typing import Any, Dict, Optional

import jwt
from flatten_dict import unflatten
from jwt.utils import base64url_decode

from sanic_jwt_extended.exceptions import JWTDecodeError
from sanic_jwt_extended.jwt_manager import JWT


@dataclass
class Token:
    raw_jwt: str
    raw_data: Dict[str, Any] = field(init=False)

    # Metadata
    type: str = field(init=False)
    role: Optional[str] = field(init=False, default=None)
    fresh: Optional[bool] = field(init=False, default=None)
    identity: str = field(init=False)

    # Registered claims
    iss: Optional[str] = field(init=False, default=None)
    sub: str = field(init=False)
    aud: Optional[str] = field(init=False, default=None)
    exp: Optional[datetime.datetime] = field(init=False, default=None)
    nbf: datetime.datetime = field(init=False)
    iat: datetime.datetime = field(init=False)
    jti: uuid.UUID = field(init=False)

    # Additional claims
    public_claims: Dict[str, Any] = field(init=False, default=None)
    private_claims: Dict[str, Any] = field(init=False, default=None)

    def __post_init__(self):
        self.raw_data = self._decode_jwt()

        self.type = self._get_type()
        self.role = self.raw_data.get("role") if JWT.config.use_acl else None
        self.fresh = self.raw_data.get("fresh") if self.type == "access" else None

        try:
            self.iss = self.raw_data.get("iss")
            self.sub = self.identity = self.raw_data["sub"]
            self.aud = self.raw_data.get("aud")
            exp = self.raw_data.get("exp")
            nbf = self.raw_data["nbf"]
            iat = self.raw_data["iat"]
            jti = self.raw_data["jti"]
        except KeyError as e:
            raise JWTDecodeError(
                f"Can not get registered claims from payload. missing {e}"
            )

        try:
            self.jti = uuid.UUID(jti)
        except Exception:
            raise JWTDecodeError(f"Wrong jti")

        try:
            self.exp = datetime.datetime.utcfromtimestamp(exp) if exp else None
            self.nbf = datetime.datetime.utcfromtimestamp(nbf)
            self.iat = datetime.datetime.utcfromtimestamp(iat)
        except Exception:
            raise JWTDecodeError(f"Wrong timestamp for 'nbf' or/and 'iat'")

        self.public_claims = self._get_public_claims() if JWT.config.public_claim_namespace else {}
        self.private_claims = self._get_private_claims()

    def _get_private_claims(self):
        private_claims = {
            k.replace(f"{JWT.config.private_claim_prefix}.", ""): v
            for k, v in self.raw_data.items()
            if k.startswith(JWT.config.private_claim_prefix)
            and not k.startswith(JWT.config.public_claim_namespace)
            and k not in ("iss", "sub", "aud", "exp", "nbf", "iat", "jti")
        }
        return private_claims

    def _get_public_claims(self):
        public_claims = {
            k.replace(JWT.config.public_claim_namespace, ""): v
            for k, v in self.raw_data.items()
            if k.startswith(JWT.config.public_claim_namespace)
            and k not in ("iss", "sub", "aud", "exp", "nbf", "iat", "jti")
        }

        return unflatten(public_claims, splitter="path")

    def _get_type(self):
        raw_header = self.raw_jwt.split(".")[0]
        header: Dict[str, str] = json.loads(base64url_decode(raw_header))

        if header.get("class") not in ("access", "refresh"):
            raise JWTDecodeError(
                "Can not resolve token type by JOSE header. missing 'class'"
            )

        return header.get("class")

    def _decode_jwt(self):
        algorithm = JWT.config.algorithm
        secret = (
            JWT.config.secret_key
            if algorithm.startswith("HS")
            else JWT.config.public_key
        )

        jwt_data = jwt.decode(self.raw_jwt, secret, algorithms=[algorithm])

        return jwt_data



from functools import wraps
from typing import Callable, List, Optional

from sanic.request import Request

from sanic_jwt_extended.exceptions import (
    AccessDeniedError,
    ConfigurationConflictError,
    FreshTokenRequiredError,
    InvalidHeaderError,
    NoAuthorizationError,
    RevokedTokenError,
    WrongTokenError,
)
from sanic_jwt_extended.jwt_manager import JWT
from sanic_jwt_extended.tokens import Token


def _get_request(args) -> Request:
    if isinstance(args[0], Request):
        request = args[0]
    else:
        request = args[1]
    return request


def _get_raw_jwt_from_request(request: Request) -> str:
    functions: List[Callable[[Request], str]] = []

    for eligible_location in JWT.config.token_location:
        if eligible_location == "header":
            functions.append(_get_raw_jwt_from_headers)
        if eligible_location == "query":
            functions.append(_get_raw_jwt_from_query_params)

    raw_jwt = None
    errors = []

    for f in functions:
        try:
            raw_jwt = f(request)
            break
        except NoAuthorizationError as e:
            errors.append(str(e))

    if not raw_jwt:
        raise NoAuthorizationError(', '.join(errors))

    return raw_jwt


def _get_raw_jwt_from_headers(request):
    header_key = JWT.config.jwt_header_key
    header_prefix = JWT.config.jwt_header_prefix

    token_header = request.headers.get(header_key)

    if not token_header:
        raise NoAuthorizationError(f"Missing {header_key} header")

    parts: List[str] = token_header.split()

    if parts[0] != header_prefix or len(parts) != 2:
        raise InvalidHeaderError(
            f"Bad {header_key} header. Expected value '{header_prefix} <JWT>'"
        )

    encoded_token: str = parts[1]

    return encoded_token


def _get_raw_jwt_from_query_params(request):
    encoded_token = request.args.get(JWT.config.jwt_query_param_name)
    if not encoded_token:
        raise NoAuthorizationError(
            f"Missing {JWT.config.jwt_query_param_name} query parameter"
        )

    return encoded_token


def _get_raw_jwt_from_cookies(request):
    pass


def jwt_required(
    function=None, *, allow=None, deny=None, fresh_required=False,
):
    def real(fn):
        @wraps(fn)
        async def wrapper(*args, **kwargs):
            request = _get_request(args)
            raw_jwt = _get_raw_jwt_from_request(request)

            token_obj = Token(raw_jwt)

            if token_obj.type != "access":
                raise WrongTokenError("Only access tokens are allowed")

            if fresh_required and not token_obj.fresh:
                raise FreshTokenRequiredError("Only fresh access tokens are allowed")

            if allow and token_obj.role not in allow:
                raise AccessDeniedError("You are not allowed to access here")

            if deny and token_obj.role in deny:
                raise AccessDeniedError("You are not allowed to access here")

            if JWT.config.use_blacklist and JWT.blacklist.is_blacklisted(token_obj):
                raise RevokedTokenError("Token has been revoked")

            kwargs["token"] = token_obj

            return await fn(*args, **kwargs)

        return wrapper

    if function:
        return real(function)
    else:
        if allow and deny:
            raise ConfigurationConflictError(
                "Can not use 'deny' and 'allow' option together."
            )
        return real


def jwt_optional(function):
    @wraps(function)
    async def wrapper(*args, **kwargs):
        request = _get_request(args)
        token_obj: Optional[Token] = None

        try:
            raw_jwt = _get_raw_jwt_from_request(request)

            token_obj = Token(raw_jwt)

            if token_obj.type != "access":
                raise WrongTokenError("Only access tokens are allowed")
        except NoAuthorizationError:
            pass

        kwargs["token"] = token_obj

        return await function(*args, **kwargs)

    return wrapper


def refresh_jwt_required(function=None, *, allow=None, deny=None):
    def real(fn):
        @wraps(fn)
        async def wrapper(*args, **kwargs):
            request = _get_request(args)
            raw_jwt = _get_raw_jwt_from_request(request)

            token_obj = Token(raw_jwt)

            if token_obj.type != "refresh":
                raise WrongTokenError("Only refresh tokens are allowed")

            if allow and token_obj.role not in allow:
                raise AccessDeniedError("You are not allowed to refresh in here")

            if deny and token_obj.role in deny:
                raise AccessDeniedError("You are not allowed to refresh in here")

            if JWT.config.use_blacklist and JWT.blacklist.is_blacklisted(token_obj):
                raise RevokedTokenError("Token has been revoked")

            kwargs["token"] = token_obj

            return await fn(*args, **kwargs)

        return wrapper

    if function:
        return real(function)
    else:
        if allow and deny:
            raise ConfigurationConflictError(
                "Can not use 'deny' and 'allow' option together."
            )
        return real



# Stubs for tokens (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import datetime
import uuid
from typing import Any, Dict, Optional

class Token:
    raw_jwt: str
    raw_data: Dict[str, Any] = ...
    type: str = ...
    role: Optional[str] = ...
    fresh: Optional[bool] = ...
    identity: str = ...
    iss: Optional[str] = ...
    sub: str = ...
    aud: Optional[str] = ...
    exp: Optional[datetime.datetime] = ...
    nbf: datetime.datetime = ...
    iat: datetime.datetime = ...
    jti: uuid.UUID = ...
    public_claims: Dict[str, Any] = ...
    private_claims: Dict[str, Any] = ...
    def __post_init__(self) -> None: ...
    def _get_private_claims(self) -> Dict[str, Any]: ...
    def _get_public_claims(self) -> Dict[str, Any]: ...
    def _get_type(self) -> str: ...
    def _decode_jwt(self) -> Dict[str, Any]: ...
    def __init__(self, raw_jwt: Any) -> None: ...



import datetime
import uuid
import warnings
from contextlib import contextmanager

import jwt
from flatten_dict import flatten
from jwt import ExpiredSignatureError, InvalidTokenError

from sanic_jwt_extended.blacklist import InMemoryBlacklist
from sanic_jwt_extended.config import Config
from sanic_jwt_extended.exceptions import (
    AccessDeniedError,
    ConfigurationConflictError,
    FreshTokenRequiredError,
    InvalidHeaderError,
    JWTDecodeError,
    NoAuthorizationError,
    RevokedTokenError,
    WrongTokenError,
)
from sanic_jwt_extended.handler import Handler


class JWT:
    config = None
    handler = None
    blacklist = None

    @classmethod
    @contextmanager
    def initialize(cls, app):
        cls.config = Config()
        cls.handler = Handler()

        yield JWT

        cls.config.read_only = True
        cls.handler.read_only = True
        cls._validate_config()
        cls._setup_blacklist()
        cls._set_error_handlers(app)

    @classmethod
    def _setup_blacklist(cls):
        if cls.config.use_blacklist is True:
            blacklist_cls = (
                cls.config.blacklist_class
                if cls.config.blacklist_class
                else InMemoryBlacklist
            )

            cls.blacklist = blacklist_cls()

    @classmethod
    def _validate_config(cls):
        if cls.config.algorithm.startswith("HS") and not cls.config.secret_key:
            raise ConfigurationConflictError(
                "HS* algorithm needs secret key to encode token"
            )
        elif cls.config.algorithm.startswith("RS"):
            if not cls.config.private_key:
                raise ConfigurationConflictError(
                    "RS* algorithm needs private key to encode token"
                )
            if not cls.config.public_key:
                raise ConfigurationConflictError("RS* algorithm needs public key")

        if cls.config.use_blacklist:
            if not cls.config.blacklist_class:
                warnings.warn(
                    "Blacklist enabled but blacklist class was not specified. "
                    "Falling back to default in-memory blacklist"
                )

    @classmethod
    def _set_error_handlers(cls, app):
        app.error_handler.add(NoAuthorizationError, cls.handler.no_authorization)
        app.error_handler.add(ExpiredSignatureError, cls.handler.expired_signature)
        app.error_handler.add(InvalidHeaderError, cls.handler.invalid_header)
        app.error_handler.add(InvalidTokenError, cls.handler.invalid_token)
        app.error_handler.add(JWTDecodeError, cls.handler.jwt_decode_error)
        app.error_handler.add(WrongTokenError, cls.handler.wrong_token)
        app.error_handler.add(RevokedTokenError, cls.handler.revoked_token)
        app.error_handler.add(FreshTokenRequiredError, cls.handler.fresh_token_required)
        app.error_handler.add(AccessDeniedError, cls.handler.access_denied)

    @classmethod
    def _encode_jwt(cls, token_type, payload, expires_delta):
        algorithm = cls.config.algorithm
        secret = (
            cls.config.secret_key
            if algorithm.startswith("HS")
            else cls.config.private_key
        )

        iss = payload.pop("iss") if payload.get("iss") else cls.config.default_iss
        aud = payload.pop("aud") if payload.get("aud") else cls.config.default_aud
        iat = datetime.datetime.utcnow()
        nbf = payload.pop("nbf") if payload.get("nbf") else iat
        jti = uuid.uuid4().hex

        reserved_claims = {"iss": iss, "aud": aud, "jti": jti, "iat": iat, "nbf": nbf}

        if isinstance(expires_delta, datetime.timedelta):
            reserved_claims["exp"] = iat + expires_delta

        payload.update(reserved_claims)
        payload = {k: v for k, v in payload.items() if v is not None}

        header = {"class": token_type}

        token = jwt.encode(
            payload, secret, algorithm, header, cls.config.json_encoder
        ).decode("utf-8")

        return token

    @classmethod
    def create_access_token(
        cls,
        identity,
        role=None,
        fresh=None,
        *,
        expires_delta=None,
        public_claims=None,
        private_claims=None,
        iss=None,
        aud=None,
        nbf=None,
    ):
        private_claim_prefix = (
            cls.config.private_claim_prefix if cls.config.private_claim_prefix else ""
        )
        payload = {"iss": iss, "sub": identity, "aud": aud, "nbf": nbf}

        if role:
            if not cls.config.use_acl:
                raise ConfigurationConflictError("You should enable ACL to use.")
            payload[cls.config.acl_claim] = role

        if fresh is not None and isinstance(fresh, bool):
            payload["fresh"] = fresh

        if public_claims:
            if not cls.config.public_claim_namespace:
                raise ConfigurationConflictError(
                    "You should specify namespace to use public claims. "
                    "\n find more at: https://auth0.com/docs/tokens/concepts/claims-namespacing"
                )

            public_claims = flatten(public_claims, reducer="path")
            for k, v in public_claims.items():
                payload[cls.config.public_claim_namespace + k] = v

        if private_claims:
            for k, v in private_claims:
                payload[private_claim_prefix + k] = v

        if expires_delta is None:
            expires_delta = cls.config.access_token_expires

        access_token = cls._encode_jwt("access", payload, expires_delta)

        return access_token

    @classmethod
    def create_refresh_token(
        cls,
        identity,
        role=None,
        *,
        expires_delta=None,
        public_claims=None,
        private_claims=None,
        iss=None,
        aud=None,
        nbf=None,
    ):
        private_claim_prefix = (
            cls.config.private_claim_prefix if cls.config.private_claim_prefix else ""
        )
        payload = {"iss": iss, "sub": identity, "aud": aud, "nbf": nbf}

        if role:
            if not cls.config.use_acl:
                raise ConfigurationConflictError("You should enable ACL to use.")
            payload[cls.config.acl_claim] = role

        if public_claims:
            if not cls.config.public_claim_namespace:
                raise ConfigurationConflictError(
                    "You should specify namespace to use public claims. "
                    "\n find more at: https://auth0.com/docs/tokens/concepts/claims-namespacing"
                )

            public_claims = flatten(public_claims, reducer="path")
            for k, v in public_claims.items():
                payload[cls.config.public_claim_namespace + k] = v

        if private_claims:
            for k, v in private_claims:
                payload[f"{private_claim_prefix}.{k}"] = v

        if expires_delta is None:
            expires_delta = cls.config.access_token_expires

        refresh_token = cls._encode_jwt("refresh", payload, expires_delta)

        return refresh_token

    @classmethod
    async def revoke(cls, token):
        if not cls.config.use_blacklist:
            raise ConfigurationConflictError(
                "To revoke token, you should enable blacklist"
            )
        cls.blacklist.register(token)



# Stubs for blacklist (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import abc
from abc import ABC, abstractmethod
from typing import Any

from sanic_jwt_extended.tokens import Token

class BlacklistABC(ABC, metaclass=abc.ABCMeta):
    @abstractmethod
    def register(self, token: Token) -> None: ...
    @abstractmethod
    def is_blacklisted(self, token: Token) -> bool: ...

class InMemoryBlacklist(BlacklistABC):
    blacklist: Any = ...
    def __init__(self) -> None: ...
    def register(self, token: Token) -> None: ...
    def is_blacklisted(self, token: Token) -> bool: ...

class RedisBlacklist(BlacklistABC):
    def register(self, token: Token) -> None: ...
    def is_blacklisted(self, token: Token) -> bool: ...



from typing import Any, Callable, List, Tuple

from sanic.request import Request

def _get_request(args: Tuple[Any]) -> Request: ...
def _get_raw_jwt_from_request(request: Request) -> str: ...
def _get_raw_jwt_from_headers(request: Request) -> str: ...
def _get_raw_jwt_from_query_params(request: Request) -> str: ...
def _get_raw_jwt_from_cookies(request: Request) -> str: ...
def jwt_required(
    function: Callable = ...,
    *,
    allow: List[str] = ...,
    deny: List[str] = ...,
    fresh_required: bool = ...
) -> Any: ...
def jwt_optional(function: Callable) -> Any: ...
def refresh_jwt_required(
    function: Callable = ..., *, allow: List[str] = ..., deny: List[str] = ...
) -> Any: ...



# Stubs for config (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from datetime import timedelta
from typing import Any, Optional, Tuple, Type, Union

from sanic_jwt_extended.blacklist import BlacklistABC

class Config:
    read_only: bool = ...
    secret_key: Optional[str] = ...
    public_key: Optional[str] = ...
    private_key: Optional[str] = ...
    default_iss: Optional[str] = ...
    default_aud: Optional[str] = ...
    json_encoder: Any = ...
    token_location: Tuple[str] = ...
    access_token_expires: Union[timedelta, bool] = ...
    refresh_token_expires: Union[timedelta, bool] = ...
    algorithm: str = ...
    public_claim_namespace: str = ...
    private_claim_prefix: str = ...
    jwt_header_key: str = ...
    jwt_header_prefix: str = ...
    jwt_query_param_name: str = ...
    use_acl: bool = ...
    acl_claim: str = ...
    use_blacklist: bool = ...
    blacklist_class: Optional[Type[BlacklistABC]] = ...
    def __setattr__(self, key: Any, value: Any) -> None: ...
    def __init__(
        self,
        read_only: Optional[str] = ...,
        secret_key: Optional[str] = ...,
        public_key: Optional[str] = ...,
        private_key: Optional[str] = ...,
        default_iss: Optional[str] = ...,
        default_aud: Optional[str] = ...,
        json_encoder: Any = ...,
        token_location: Tuple[str] = ...,
        access_token_expires: Union[timedelta, bool] = ...,
        refresh_token_expires: Union[timedelta, bool] = ...,
        algorithm: Optional[str] = ...,
        public_claim_namespace: Optional[str] = ...,
        private_claim_prefix: Optional[str] = ...,
        jwt_header_key: Optional[str] = ...,
        jwt_header_prefix: Optional[str] = ...,
        use_acl: Optional[str] = ...,
        acl_claim: Optional[str] = ...,
        use_blacklist: Optional[str] = ...,
        blacklist_class: Optional[Type[BlacklistABC]] = ...,
    ) -> None: ...



class JWTExtendedException(Exception):
    """
    Base except which all sanic_jwt_extended errors extend
    """

    pass


class JWTDecodeError(JWTExtendedException):
    """
    An error decoding a JWT
    """

    pass


class InvalidHeaderError(JWTExtendedException):
    """
    An error getting header information from a request
    """

    pass


class NoAuthorizationError(JWTExtendedException):
    """
    An error raised when no authorization token was found in a protected endpoint
    """

    pass


class WrongTokenError(JWTExtendedException):
    """
    Error raised when attempting to use a refresh token to access an endpoint
    or vice versa
    """

    pass


class RevokedTokenError(JWTExtendedException):
    """
    Error raised when a revoked token attempt to access a protected endpoint
    """

    pass


class FreshTokenRequiredError(JWTExtendedException):
    """
    Error raised when a valid, non-fresh JWT attempt to access an endpoint
    protected by jwt_required with fresh_required is True
    """

    pass


class AccessDeniedError(JWTExtendedException):
    """
    Error raised when a valid JWT attempt to access an endpoint
    protected by jwt_required with not allowed role
    """

    pass


class ConfigurationConflictError(JWTExtendedException):
    """
    Error raised when trying to use allow and deny option together in jwt_required
    """

    pass



from dataclasses import dataclass
from datetime import timedelta
from json import JSONDecoder, JSONEncoder
from typing import Any, Optional, Tuple, Type, Union

from sanic_jwt_extended.blacklist import BlacklistABC


@dataclass
class Config:
    read_only: bool = False

    # JWT secrets
    secret_key: Optional[str] = None

    public_key: Optional[str] = None
    private_key: Optional[str] = None

    # Default values for reserved claims
    default_iss: Optional[str] = None
    default_aud: Optional[str] = None

    # General configs
    json_encoder: Any = JSONEncoder
    token_location: Tuple[str] = ("header",)
    access_token_expires: Union[timedelta, bool] = timedelta(minutes=15)
    refresh_token_expires: Union[timedelta, bool] = timedelta(days=30)
    algorithm: str = "HS256"

    public_claim_namespace: str = ""  # should be URL
    private_claim_prefix: str = ""

    # JWT in headers configs
    jwt_header_key: str = "Authorization"
    jwt_header_prefix: str = "Bearer"

    # JWT in query params option
    jwt_query_param_name: str = "jwt"

    # ACL config
    use_acl: bool = False
    acl_claim: str = "permission"

    # Blacklist config
    use_blacklist: bool = False
    blacklist_class: Optional[Type[BlacklistABC]] = None

    def __setattr__(self, key, value):
        if self.read_only:
            raise RuntimeError("Can not set attribute after app initialized.")
        super().__setattr__(key, value)



import datetime
from typing import Any, ContextManager, Dict, Optional

from sanic import Sanic

from sanic_jwt_extended.blacklist import BlacklistABC
from sanic_jwt_extended.config import Config
from sanic_jwt_extended.handler import Handler
from sanic_jwt_extended.tokens import Token

class JWT:
    config: Config = ...
    handler: Handler = ...
    blacklist: Optional[BlacklistABC] = ...
    @classmethod
    def initialize(cls: JWT, app: Sanic) -> ContextManager[JWT]: ...
    @classmethod
    def _setup_blacklist(cls: JWT): ...
    @classmethod
    def _validate_config(cls: JWT): ...
    @classmethod
    def _set_error_handlers(cls: JWT, app: Sanic) -> None: ...
    @classmethod
    def _encode_jwt(
        cls: JWT,
        token_type: str,
        payload: Dict[str, Any],
        expires_delta: datetime.timedelta,
    ) -> str: ...
    @classmethod
    def create_access_token(
        cls: JWT,
        identity: str,
        role: str = ...,
        fresh: bool = ...,
        *,
        expires_delta: datetime.timedelta = ...,
        public_claims: Dict[str, Any] = ...,
        private_claims: Dict[str, Any] = ...,
        iss: str = ...,
        aud: str = ...,
        nbf: datetime.datetime = ...,
    ) -> str: ...
    @classmethod
    def create_refresh_token(
        cls: JWT,
        identity: str,
        role: str = ...,
        *,
        expires_delta: datetime.timedelta = ...,
        public_claims: Dict[str, Any] = ...,
        private_claims: Dict[str, Any] = ...,
        iss: str = ...,
        aud: str = ...,
        nbf: datetime.datetime = ...,
    ) -> str: ...
    @classmethod
    async def revoke(cls: JWT, token: Token): ...



import warnings
from abc import ABC, abstractmethod


class BlacklistABC(ABC):
    @abstractmethod
    def register(self, token):
        pass

    @abstractmethod
    def is_blacklisted(self, token):
        pass


class InMemoryBlacklist(BlacklistABC):
    def __init__(self):
        self.blacklist = []
        warnings.warn(
            "Using in-memory blacklist is not recommended for production environment"
        )

    def register(self, token):
        self.blacklist.append(token.jti)

    def is_blacklisted(self, token):
        return token.jti in self.blacklist


class RedisBlacklist(BlacklistABC):  # TODO implement
    def register(self, token):
        pass

    def is_blacklisted(self, token):
        pass



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['sanic_jwt_extended']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT>=1.6.4,<2.0.0', 'flatten-dict>=0.2.0,<0.3.0', 'sanic>=18.12.0']

extras_require = \
{':python_version >= "3.6.0" and python_version < "3.7.0"': ['dataclasses']}

setup_kwargs = {
    'name': 'sanic-jwt-extended',
    'version': '1.0.dev4',
    'description': 'Extended JWT integration with Sanic',
    'long_description': '<h1 align="center">🛡 Sanic-JWT-Extended 🛡</h1>\n\n<div align="center"> \n\n[![Downloads](https://pepy.tech/badge/sanic-jwt-extended)](https://pepy.tech/project/sanic-jwt-extended)\n![PyPI](https://img.shields.io/pypi/v/sanic-jwt-extended.svg)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/sanic-jwt-extended.svg)\n![code style](https://img.shields.io/badge/code%20style-black-black.svg)\n[![Documentation Status](https://api.netlify.com/api/v1/badges/c2cf1ea1-bae1-448f-b52c-0dea6516446a/deploy-status)](https://sanic-jwt-extended.seonghyeon.dev)\n\n</div>\n\n> **☢️\xa0This is README of 1.0.dev version. [Click here](https://github.com/NovemberOscar/Sanic-JWT-Extended/tree/v0.4.4) to checkout current stable version(v0.4.4)**\n\n## 🚀 What is Sanic-JWT-Extended?\nSanic-JWT-Extended is an open source Sanic extension that provides JWT support (comply with RFC standard)\n\n## 💡 Why Sanic-JWT-Extended?\nSanic-JWT-Extended not only adds support for using JSON Web Tokens (JWT) to Sanic for protecting views,\nbut also many helpful (and **optional**) features  built in to make working with JSON Web Tokens\neasier. These include:\n\n* Support for adding public claims with [namespacing](https://auth0.com/docs/tokens/concepts/claims-namespacing)\n* Support for adding private claims\n* [Refresh tokens](https://auth0.com/blog/refresh-tokens-what-are-they-and-when-to-use-them/)\n* Token freshness and separate view decorators to only allow fresh tokens\n* Access control\n* blacklist support with some built-in blacklist\n* Provides Token object for easier jwt manifulation\n\n## ⚡️ Installation\n```shell script\n$ pip install sanic-jwt-extended --pre\n```\n```shell script\n$ poetry add sanic-jwt-extended --git https://github.com/NovemberOscar/Sanic-JWT-Extended.git\n```\n```shell script\n$ pipenv install sanic-jwt-extended --pre\n```\n\n## 📚 Documentation\n<a href="https://sanic-jwt-extended.seonghyeon.dev">\n<img src="https://i.imgur.com/eXRmcKO.png)](https://sanic-jwt-extended.seonghyeon.dev/" width="300" />\n</a>\n\n\n## 🛠 Developing Sanic-JWT-Extended\n\n### Prerequesties\n- [poetry](https://github.com/sdispater/poetry)\n\n### Installaion\n```shell script\n$ make env\n```\nthis will install dependencies with poetry. if poetry not found, will install poetry.\n\n### Development\n- `make format`: this will format your code with `isort` and `black`\n- `make check`: this will lint your code with `isort`, `black`, and `pylint`\n- `make clean`: this will remove temporary things.\n\n### Commit Convention\n```\n<{verb}>({scope}): {summary}\n```\n\n### Testing\n- **TBD**\n',
    'author': 'Seonghyeon Kim',
    'author_email': 'kim@seonghyeon.dev',
    'url': 'https://github.com/NovemberOscar/Sanic-JWT-Extended',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
